#==================================================================================================
# Imports and defines
#==================================================================================================
import bpy
from bpy.props import *
#==================================================================================================
# 
#==================================================================================================






#==================================================================================================
# Addon Info
#==================================================================================================
bl_info = {
    "name": "Add Toon Outline",
    "description": "",
    "author": "Daitomodachi",
    "version": (1, 0, 0),
    "blender": (2, 81, 0),
    "location": "Operator",
    "warning": "",
    "wiki_url": "",
    "tracker_url": "",
    "support": "COMMUNITY",
    "category": "Material/Modeling"
}
#==================================================================================================
# 
#==================================================================================================                       
                       




#==================================================================================================
# Outline Function
#==================================================================================================
def outline_mesh(thickness):
    # Creates the Outline Material
    mat = bpy.data.materials.new("My Outline")
    
    # Enables Nodes
    mat.use_nodes = True

    # Stores the material's node tree
    material_list = mat.node_tree.nodes

    # Creates the emission node and deletes the Principled BSDF Node
    mat.node_tree.nodes.new('ShaderNodeEmission')
    Emission_Node = mat.node_tree.nodes['Emission']
    Material_Output_Node = mat.node_tree.nodes['Material Output']
    mat.node_tree.nodes.remove(mat.node_tree.nodes['Principled BSDF'])

    # Connects the Emission Node and the Material Output nodes together
    mat.node_tree.links.new(Material_Output_Node.inputs[0], Emission_Node.outputs[0])
    Emission_Node.inputs[0].default_value = (0,0,0,1)
    
    # Sets the necessary parameters for the material
    mat.use_backface_culling = True
    mat.blend_method = 'BLEND'
    mat.shadow_method = 'NONE'
    
    

    #Creates the solidify modifier and sets the parameters for it
    objs =  bpy.context.selected_objects
    
    for o in objs:
        if o.type == "MESH":
            # Appends the material to the current mesh selected
            mesh = o.data.materials.append(mat)

            mod = o.modifiers.new("Solidify", 'SOLIDIFY')
            mod.use_flip_normals = True
            mod.use_rim = False
            mod.use_rim_only = True
            mod.material_offset = 100
            mod.thickness = thickness
#==================================================================================================
# 
#==================================================================================================


                    



#==================================================================================================
# Add Outline Operator Class
#==================================================================================================                       
class Add_Outline(bpy.types.Operator):
    #BL Info for the Operator
    bl_idname = "object.add_outline"
    bl_label = "Add Outline To Mesh"
    bl_property = "object_outliner"
    
    
    # Creates the Thickness Property to handle the thickness parameter of the modifier
    Thickness: bpy.props.FloatProperty(name="Thickness")

   
    # Runs the operator 
    def execute(self, context):
        outline_mesh(self.Thickness)
        self.report({'INFO'}, "Added an outline to the mesh")
        return {'FINISHED'}
    
    # Invokes the command when called
    def invoke(self, context, event):
        wm = context.window_manager
        return wm.invoke_props_dialog(self)

    # Puts the UI on screen
    def draw(self, context):
        layout = self.layout
        col = layout.column()


        row = col.row()
        row.prop(self, "Thickness")

#==================================================================================================
# 
#==================================================================================================






#==================================================================================================
# Register and Unregister classes
#==================================================================================================     
# Stores the classes used in the code
classes = [Add_Outline]  

def register():
    for cls in classes:
        bpy.utils.register_class(cls)

def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)

if __name__ == "__main__":
    register()
#==================================================================================================
# 
#==================================================================================================